package archiveModels

//go:generate reform --gofmt=false

import (
	asuModels "devel.mephi.ru/dyokunev/go-asu-models"
	I "devel.mephi.ru/dyokunev/wwwvoip/app/iface"
	"encoding/json"
	"github.com/revel/revel"
	"github.com/xaionaro-go/extime"
)

//reform:appointment
type Appointment struct {
	Id              int                `json:"id" reform:"id,pk" view:"omit"`
	StfCardId       int                `json:"stf_card_id" reform:"stf_card_id" sql:"index"`
	Name            string             `json:"name" reform:"name"`
	Description     string             `json:"description" reform:"description"`
	SubscriberKey   int64              `json:"subscriber_key" reform:"subscriber_key"`
	Subscriber      I.UserBase         `json:"-" reform:"-"`
	UnitKey         string             `json:"unit_key" reform:"unit_key"`
	SecurityLevel0  int                `json:"security_level_0" reform:"security_level_0"`
	SecurityLevel1  int                `json:"security_level_1" reform:"security_level_1"`
	SecretaryKey    int                `json:"secretary_key" reform:"secretary_key"`
	LineKey         string             `json:"line_key" reform:"line_key"`
	IsHead          bool               `json:"is_head" reform:"is_head"`
	RoomKey         string             `json:"room_key" reform:"room_key"`
	PersonnelNumber *int               `json:"personnel_number" reform:"personnel_number"`
	Formular        asuModels.Formular `json:"formular" reform:"-"`
	CreatedAt       extime.Date        `json:"created_at" reform:"created_at"`
	IsActual        bool               `json:"is_actual" reform:"is_actual" sql:"index"`
	Notes           string             `json:"notes" reform:"notes" sql:"size:1024"`
}

func (a *Appointment) AfterFind() error {
	return nil
}

func (a *Appointment) BeforeInsert() error {
	a.CreatedAt = extime.Date(extime.Now())
	return nil
}

func (a Appointment) UniqueHash() string {
	a.Id = 0
	a.IsActual = true
	a.CreatedAt = extime.Date{}
	a.Formular = asuModels.Formular{}
	a.Subscriber = nil

	b, err := json.Marshal(a)
	if err != nil {
		revel.ERROR.Panicf("Cannot JSONize an archive appointment (%v): %v", a, err.Error())
	}

	return string(b)
}
